<?php
	
	list ($channel, $start_time, $extensions) = $irc_call_args;
	
	if (!isset ($this->custom_vars->yandere_games[$channel]))
	{
		return;
	}
	
	$game = $this->custom_vars->yandere_games[$channel];
	
	if ($start_time != $game->start_time)
	{
		// This message is no longer relevant.
		return;
	}
	
	if ($game->active)
	{
		// The game already started.
		return;
	}
	
	if (count($game->players) < 4)
	{
		if ($extensions < $game->max_extensions)
		{
			output($channel, $game->s.'We don\'t have enough players yet. We need at least 4 players to start a game. I\'ll wait for another 60 seconds. Please type "!yandere" to join.'.$game->e);
			
			$this->schedule_event(time() + 60, array ('yandere/start-game', $channel, $game->start_time, $extensions + 1));
		}
		else
		{
			output($channel, $game->s.'There aren\'t enough players to start a yandere game. Try again later.'.$game->e);
			
			unset ($this->custom_vars->yandere_games[$channel]);
		}
		
		return;
	}
	
	if (is_null($game->start_mode))
	{
		$game->start_mode = count($game->players) % 2 == 1 ? 'night' : 'day';
	}
	
	if ($game->restrict_voice)
	{
		// Silence all users in the channel.
		
		$this->send('MODE '.$channel.' +m');
	}
	
	call('yandere/remove-voice', $channel);
	
	$game->active = true;
	
	// Get a random-order list of the players, to simplify random role assignment.
	
	$user_ids = array_keys($game->players);
	shuffle($user_ids);
	
	// Find out how many of each role we're going to have.
	
	$count = count($user_ids);
	
	$units = array ();
	
	$num_yanderes = floor(($count - 4) / 3) + 1;
	
	switch ($game->complexity)
	{
	case 'basic':
		$units[$this->custom_vars->yandere_standard_roles['mafioso']] = $num_yanderes;
		break;
	
	case 'normal':
		for ($i=0; $i < $num_yanderes; $i++)
		{
			$yandere_type = array_rand_weighted($this->custom_vars->yandere_yandere_odds);
			
			if (!isset ($units[$yandere_type]))
			{
				$units[$yandere_type] = 1;
			}
			else
			{
				$units[$yandere_type]++;
			}
		}
		break;
	}
	
	$units[$this->custom_vars->yandere_standard_roles['detective']] = floor($count / 14) + 1;
	
	$remaining = $count - array_sum($units);
	
	$innocent = $this->custom_vars->yandere_standard_roles['innocent'];
	
	switch ($game->complexity)
	{
	case 'basic':
		$units[$innocent] = $remaining;
		break;
	
	case 'normal':
		$role_odds = $this->custom_vars->yandere_role_odds;
		
		for ($i=0; $i < $remaining; $i++)
		{
			if ($i == $remaining - 1 && isset ($units[$innocent]) && $units[$innocent] == $i)
			{
				unset ($role_odds[$innocent]);
			}
			
			$role = array_rand_weighted($role_odds);
			
			if (!isset ($units[$role]))
			{
				$units[$role] = 1;
			}
			else
			{
				$units[$role]++;
			}
		}
		break;
	}
	
	foreach ($units as $key => $value)
	{
		if ($value == 0)
		{
			unset ($units[$key]);
		}
	}
	
	// Assign the roles.
	
	$unit_list = array ();
	
	$yandere_nicks = array ();
	$yandere_users = array ();
	
	$i = 0;
	
	foreach ($units as $role => $num)
	{
		if (!$game->blind_roles)
		{
			$unit_list[] = $num.' '.str_plural($role, $num, $this->custom_vars->yandere_role_plurals[$role]);
		}
		
		for ($j=0; $j < $num; $j++)
		{
			$game->players[$user_ids[$i]]->role = $role;
			
			if (isset ($this->custom_vars->yandere_yandere_odds[$role]))
			{
				$user = $game->players[$user_ids[$i]];
				$yandere_users[] = $user;
				$yandere_nicks[] = $user->nick;
			}
			
			$i++;
		}
	}
	
	if ($game->blind_roles)
	{
		$role = $this->custom_vars->yandere_standard_roles['mafioso'];
		
		$unit_list = array ($num_yanderes.' '.str_plural($role, $num_yanderes, $this->custom_vars->yandere_role_plurals[$role]));
	}
	
	$game->init_players = $game->players;
	
	foreach ($game->players as $player)
	{
		notice($player->nick, $game->s.'You\'re a '.$player->role.'. '.$this->custom_vars->yandere_role_descriptions[$player->role].$game->e);
		
		if (isset ($this->custom_vars->yandere_yandere_odds[$player->role]))
		{
			notice($player->nick, $game->s.'The yanderes in this game are: '.implode_comma_series($yandere_nicks).'.'.$game->e);
		}
	}
	
	// Introduction message
	
	$description1 = $this->custom_vars->yandere_murder_descriptions[mt_rand(0, count($this->custom_vars->yandere_murder_descriptions) - 1)];
	
	$description1 .= ' '.$this->custom_vars->yandere_weapons[mt_rand(0, count($this->custom_vars->yandere_weapons) - 1)].'. ';
	
	switch (mt_rand(0, 2))
	{
	case 0:
		$description2 = 'This looks like the work of a yandere!';
		break;
	
	case 1:
		$description2 = 'There\'s only one explanation: there is a yandere among us!';
		break;
	
	case 2:
		$description2 = 'She seems to have been the victim of a yandere!';
		break;
	}
	
	switch (mt_rand(0, 12))
	{
	case 0:
		output($channel, $game->s.'TAIHEN!! The loli village wakes up one morning to a horrible surprise: Shin-chan '.$description1.$description2.$game->e);
		break;
	
	case 1:
		output($channel, $game->s.'All the little lolis go to school one day and notice that Shin-chan seems to be missing. After school, you go to her house to check on her, and find that she '.$description1.$description2.$game->e);
		break;
	
	case 2:
		output($channel, $game->s.'At '.mt_rand(2, 4).':'.str_pad(mt_rand(0, 59), 2, '0', STR_PAD_LEFT).' in the morning, the neighborhood lolis wake up to a scream. It sounds like Shin-chan! You all run to her room and find that she '.$description1.$description2.$game->e);
		break;
	
	case 3:
		output($channel, $game->s.'Today is Shin-chan\'s birthday! And she\'s having ichigo cake! All the townslolis eagerly come to her house, but they\'re met with a surprise: Shin-chan '.$description1.$description2.$game->e);
		break;
	
	case 4:
		output($channel, $game->s.'Shin-chan is a quiet, moe loli. But she\'s especially quiet today, because she '.$description1.$description2.$game->e);
		break;
	
	case 5:
		output($channel, $game->s.'It\'s Christmas morning! Shin-chan rushes into the living room and sees a great big present with her name on it. Full of excitement, she rips open the box and finds some sort of weird alarm clock attached to several red sticks. It proceeds to explode. '.$description2.$game->e);
		break;
	
	case 6:
		output($channel, $game->s.'Shin-chan is riding merrily on the bus on her way to school. Little does she know it\'s rigged with hundreds of explosives. The bus explodes in a mighty inferno, and Shin-chan bits go flying everywhere. '.$description2.$game->e);
		break;
	
	case 7:
		output($channel, $game->s.'All her life, Shin-chan had been plagued by an illness that confined her to her room. But today, she decided to fulfill her dream: to risk it and go to school like all the other lolis. But her lunch was poisoned and she died. '.$description2.$game->e);
		break;
	
	case 8:
		output($channel, $game->s.'Everybody loves Shin-chan, except for whoever set her on fire last night. Now Shin-chan is charcoal. '.$description2.$game->e);
		break;
	
	case 9:
		output($channel, $game->s.'Shin-chan is baking cookies! Everyone comes over to her house to taste her yummy treats. But where is Shin-chan? And what\'s that funny smell coming from the oven? Uh oh! It looks like it\'s Shin-chan-flavored cookies now. '.$description2.$game->e);
		break;
	
	case 10:
		output($channel, $game->s.'Shin-chan is taking a shower. She begins her daily ritual of massaging her chest, hoping to grow breasts soon so she can finally confess to that special someone. But then she screams as the water temperature dramatically rises and quickly burns off all her flesh, leaving behind a loli skeleton. '.$description2.$game->e);
		break;
	
	case 11:
		output($channel, $game->s.'Shin-chan has always been small, but she used to be taller than 1cm. At this new size, she was easily swallowed by the pet kitten she got for her birthday last week. What caused her to shrink anyway? '.$description2.$game->e);
		break;
	
	case 12:
		output($channel, $game->s.'Shin-chan opens her shoe locker and finds a love letter! She\'d never received a love letter before! It says her secret admirer will be waiting on the school rooftop. At the end of the day, she eagerly rushes to the roof, where she receives an axe to the face. '.$description2.$game->e);
		break;
	}
	
	rsort($unit_list);
	
	output($channel, $game->s.'This game has '.implode_comma_series($unit_list).'. It\'s your job to determine the '.str_plural('yandere', $units['yanderes'], 'yanderes').'.'.$game->e);
	
	call('yandere/'.$game->start_mode, $channel);
	
?>
